/**
 * |----------------------------------------------------------------------
 * | Copyright (C) Tilen Majerle, 2014
 * | Modified by Dacian Todea for the Solar BMS project 2015 (electrodacus.com)
 * | (changes involve adding 16bit SPI and using send pixel color data only and
 * |   write continue command for big speed increase many other changes
 * |   related to SBMS menu and graph).
 * |
 * | This program is free software: you can redistribute it and/or modify
 * | it under the terms of the GNU General Public License as published by
 * | the Free Software Foundation, either version 3 of the License, or
 * | any later version.
 * |
 * | This program is distributed in the hope that it will be useful,
 * | but WITHOUT ANY WARRANTY; without even the implied warranty of
 * | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * | GNU General Public License for more details.
 * |
 * | You should have received a copy of the GNU General Public License
 * | along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * |----------------------------------------------------------------------
 */
#include "SBMS_ili9341.h"
#include "bmsmenu100.h"
#include "delay.h"
/**
 * @brief  Orientation
 * @note   Used private
 */
typedef enum {
	TM_ILI9341_Landscape,
	TM_ILI9341_Portrait
} TM_ILI9341_Orientation;

/**
 * @brief  LCD options
 * @note   Used private
 */
typedef struct {
	uint16_t width;
	uint16_t height;
	TM_ILI9341_Orientation orientation; // 1 = portrait; 0 = landscape
} TM_ILI931_Options_t;


/* Pin definitions */
#define ILI9341_RST_SET				GPIO_SetBits(ILI9341_RST_PORT, ILI9341_RST_PIN)
#define ILI9341_RST_RESET			GPIO_ResetBits(ILI9341_RST_PORT, ILI9341_RST_PIN)
#define ILI9341_CS_SET				GPIO_SetBits(ILI9341_CS_PORT, ILI9341_CS_PIN)
#define ILI9341_CS_RESET			GPIO_ResetBits(ILI9341_CS_PORT, ILI9341_CS_PIN)
#define ILI9341_WRX_SET				GPIO_SetBits(ILI9341_WRX_PORT, ILI9341_WRX_PIN)
#define ILI9341_WRX_RESET			GPIO_ResetBits(ILI9341_WRX_PORT, ILI9341_WRX_PIN)

/* Private defines */
#define ILI9341_RESET				0x01
#define ILI9341_SLEEP_OUT			0x11
#define ILI9341_GAMMA				0x26
#define ILI9341_DISPLAY_OFF			0x28
#define ILI9341_DISPLAY_ON			0x29
#define ILI9341_COLUMN_ADDR			0x2A
#define ILI9341_PAGE_ADDR			0x2B
#define ILI9341_GRAM				0x2C
#define ILI9341_MAC					0x36
#define ILI9341_PIXEL_FORMAT		0x3A
#define ILI9341_WRITE_CONTINUE		0x3C //dacus important for speed
#define ILI9341_WDB					0x51
#define ILI9341_WCD					0x53
#define ILI9341_RGB_INTERFACE		0xB0
#define ILI9341_FRC					0xB1
#define ILI9341_BPC					0xB5
#define ILI9341_DFC					0xB6
#define ILI9341_POWER1				0xC0
#define ILI9341_POWER2				0xC1
#define ILI9341_VCOM1				0xC5
#define ILI9341_VCOM2				0xC7
#define ILI9341_POWERA				0xCB
#define ILI9341_POWERB				0xCF
#define ILI9341_PGAMMA				0xE0
#define ILI9341_NGAMMA				0xE1
#define ILI9341_DTCA				0xE8
#define ILI9341_DTCB				0xEA
#define ILI9341_POWER_SEQ			0xED
#define ILI9341_3GAMMA_EN			0xF2
#define ILI9341_INTERFACE			0xF6
#define ILI9341_PRC					0xF7


#define menuforeground 0xFF90
#define menubackground 0x2100
#define decforeground 0xFFFA
#define decbackground 0x3100


/* Pin functions */
uint16_t ILI9341_x;
uint16_t ILI9341_y;
TM_ILI931_Options_t ILI9341_Opts;
uint8_t ILI9341_INT_CalledFromPuts = 0;
uint16_t LCDbuffer[512];




/* Private functions */
void TM_ILI9341_InitLCD(void);
void TM_ILI9341_SendData(uint8_t data);
void TM_ILI9341_SendData16(uint16_t data);
void TM_ILI9341_SendData16_multi(uint16_t* data,uint16_t count);
void TM_ILI9341_SendCommand(uint8_t data);
void TM_ILI9341_Delay(volatile unsigned int delay);
void TM_ILI9341_SetCursorPosition(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);

//dacus just added my fast fill
void TM_ILI9341_Init() {
	/* Init WRX pin */
	TM_GPIO_Init(ILI9341_WRX_PORT, ILI9341_WRX_PIN, TM_GPIO_Mode_OUT, TM_GPIO_OType_PP, TM_GPIO_PuPd_NOPULL, TM_GPIO_Speed_Medium);

	/* Init CS pin */
	TM_GPIO_Init(ILI9341_CS_PORT, ILI9341_CS_PIN, TM_GPIO_Mode_OUT, TM_GPIO_OType_PP, TM_GPIO_PuPd_NOPULL, TM_GPIO_Speed_Medium);

	/* Init RST pin */  //dacus not used in my application.
	//TM_GPIO_Init(ILI9341_RST_PORT, ILI9341_RST_PIN, TM_GPIO_Mode_OUT, TM_GPIO_OType_PP, TM_GPIO_PuPd_UP, TM_GPIO_Speed_Low);

	/* CS high */
	ILI9341_CS_SET;


	/* Init SPI */
	TM_SPI_Init(ILI9341_SPI, ILI9341_SPI_PINS);

	/* Init LCD */
	TM_ILI9341_InitLCD();

	/* Set default settings */
	ILI9341_x = ILI9341_y = 0;
	ILI9341_Opts.width = ILI9341_WIDTH;
	ILI9341_Opts.height = ILI9341_HEIGHT;
	ILI9341_Opts.orientation = TM_ILI9341_Portrait;

	/* Fill with white color */
	//Fill(ILI9341_COLOR_WHITE);
}

void TM_ILI9341_InitLCD(void) {
	ILI9341_RST_RESET;
	ILI9341_RST_SET;

	TM_ILI9341_SendCommand(ILI9341_RESET);
	TM_ILI9341_Delay(200000);

	TM_ILI9341_SendCommand(ILI9341_POWERA);
	TM_ILI9341_SendData(0x39);
	TM_ILI9341_SendData(0x2C);
	TM_ILI9341_SendData(0x00);
	TM_ILI9341_SendData(0x34);
	TM_ILI9341_SendData(0x02);
	TM_ILI9341_SendCommand(ILI9341_POWERB);
	TM_ILI9341_SendData(0x00);
	TM_ILI9341_SendData(0xC1);
	TM_ILI9341_SendData(0x30);
	TM_ILI9341_SendCommand(ILI9341_DTCA);
	TM_ILI9341_SendData(0x85);
	TM_ILI9341_SendData(0x00);
	TM_ILI9341_SendData(0x78);
	TM_ILI9341_SendCommand(ILI9341_DTCB);
	TM_ILI9341_SendData(0x00);
	TM_ILI9341_SendData(0x00);
	TM_ILI9341_SendCommand(ILI9341_POWER_SEQ);
	TM_ILI9341_SendData(0x64);
	TM_ILI9341_SendData(0x03);
	TM_ILI9341_SendData(0x12);
	TM_ILI9341_SendData(0x81);
	TM_ILI9341_SendCommand(ILI9341_PRC);
	TM_ILI9341_SendData(0x20);
	TM_ILI9341_SendCommand(ILI9341_POWER1);
	TM_ILI9341_SendData(0x23);
	TM_ILI9341_SendCommand(ILI9341_POWER2);
	TM_ILI9341_SendData(0x10);
	TM_ILI9341_SendCommand(ILI9341_VCOM1);
	TM_ILI9341_SendData(0x3E);//3E
	TM_ILI9341_SendData(0x28);//28
	TM_ILI9341_SendCommand(ILI9341_VCOM2);
	TM_ILI9341_SendData(0x86);
	TM_ILI9341_SendCommand(ILI9341_MAC);
	TM_ILI9341_SendData(0x28);//48
	TM_ILI9341_SendCommand(ILI9341_PIXEL_FORMAT);
	TM_ILI9341_SendData(0x55);
	TM_ILI9341_SendCommand(ILI9341_FRC);
	TM_ILI9341_SendData(0x00);
	TM_ILI9341_SendData(0x13);//original 0x18 83hz set to 100hz
	TM_ILI9341_SendCommand(ILI9341_DFC);
	TM_ILI9341_SendData(0x08);
	TM_ILI9341_SendData(0x82);
	TM_ILI9341_SendData(0x27);
	TM_ILI9341_SendCommand(ILI9341_3GAMMA_EN);
	TM_ILI9341_SendData(0x00);
	TM_ILI9341_SendCommand(ILI9341_COLUMN_ADDR);
	TM_ILI9341_SendData(0x00);
	TM_ILI9341_SendData(0x00);
	TM_ILI9341_SendData(0x00);
	TM_ILI9341_SendData(0xEF);
	TM_ILI9341_SendCommand(ILI9341_PAGE_ADDR);
	TM_ILI9341_SendData(0x00);
	TM_ILI9341_SendData(0x00);
	TM_ILI9341_SendData(0x01);
	TM_ILI9341_SendData(0x3F);
	TM_ILI9341_SendCommand(ILI9341_GAMMA);
	TM_ILI9341_SendData(0x01);
	TM_ILI9341_SendCommand(ILI9341_PGAMMA);
	TM_ILI9341_SendData(0x0F);
	TM_ILI9341_SendData(0x31);
	TM_ILI9341_SendData(0x2B);
	TM_ILI9341_SendData(0x0C);
	TM_ILI9341_SendData(0x0E);
	TM_ILI9341_SendData(0x08);
	TM_ILI9341_SendData(0x4E);
	TM_ILI9341_SendData(0xF1);
	TM_ILI9341_SendData(0x37);
	TM_ILI9341_SendData(0x07);
	TM_ILI9341_SendData(0x10);
	TM_ILI9341_SendData(0x03);
	TM_ILI9341_SendData(0x0E);
	TM_ILI9341_SendData(0x09);
	TM_ILI9341_SendData(0x00);
	TM_ILI9341_SendCommand(ILI9341_NGAMMA);
	TM_ILI9341_SendData(0x00);
	TM_ILI9341_SendData(0x0E);
	TM_ILI9341_SendData(0x14);
	TM_ILI9341_SendData(0x03);
	TM_ILI9341_SendData(0x11);
	TM_ILI9341_SendData(0x07);
	TM_ILI9341_SendData(0x31);
	TM_ILI9341_SendData(0xC1);
	TM_ILI9341_SendData(0x48);
	TM_ILI9341_SendData(0x08);
	TM_ILI9341_SendData(0x0F);
	TM_ILI9341_SendData(0x0C);
	TM_ILI9341_SendData(0x31);
	TM_ILI9341_SendData(0x36);
	TM_ILI9341_SendData(0x0F);
	TM_ILI9341_SendCommand(ILI9341_SLEEP_OUT);

	TM_ILI9341_Delay(1000000);

	TM_ILI9341_SendCommand(ILI9341_DISPLAY_ON);
	TM_ILI9341_Delay(200000);
	TM_ILI9341_SendCommand(ILI9341_GRAM);

}

void TM_ILI9341_DisplayOn(void) {
	TM_ILI9341_SendCommand(ILI9341_DISPLAY_ON);
}

void TM_ILI9341_DisplayOff(void) {
	TM_ILI9341_SendCommand(ILI9341_DISPLAY_OFF);
}


void TM_ILI9341_SendCommand(uint8_t data) {

	ILI9341_WRX_RESET;
	ILI9341_CS_RESET;
	SPI_Send(ILI9341_SPI, data);
	ILI9341_CS_SET;
}

void TM_ILI9341_SendData(uint8_t data) {
	ILI9341_WRX_SET;
	ILI9341_CS_RESET;
	SPI_Send(ILI9341_SPI, data);
	ILI9341_CS_SET;
}

//dacus
void TM_ILI9341_SendData16(uint16_t data) {
	ILI9341_WRX_SET;
	ILI9341_CS_RESET;
	TM_SPI_Send16(ILI9341_SPI, (data << 8)|(data >> 8));
	ILI9341_CS_SET;
}


//dacus
void TM_ILI9341_SendData16_multi(uint16_t* data,uint16_t count) {

	ILI9341_WRX_SET;
	ILI9341_CS_RESET;
	//TM_SPI_SendMulti16(ILI9341_SPI,data ,data,count);
	TM_SPI_WriteMulti16(ILI9341_SPI,data,count);
	ILI9341_CS_SET;

}


//dacus 16bit
void TM_ILI9341_DrawPixel(uint16_t x, uint16_t y, uint32_t color) {
	TM_ILI9341_SetCursorPosition(x, y, x, y);
	TM_ILI9341_SendCommand(ILI9341_GRAM);
	TM_ILI9341_SendData16(color);
	//LCDbuffer[0]=(color << 8)|(color >> 8);
	//TM_ILI9341_SendData16_multi(LCDbuffer,1);

}

//dacus 16bit
void TM_ILI9341_SetCursorPosition(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2) {
	TM_ILI9341_SendCommand(ILI9341_COLUMN_ADDR);
	TM_ILI9341_SendData16(x1);
	TM_ILI9341_SendData16(x2);
	TM_ILI9341_SendCommand(ILI9341_PAGE_ADDR);
	TM_ILI9341_SendData16(y1);
	TM_ILI9341_SendData16(y2);

}

//dacus just a test
void LCD_invert(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2,uint8_t invert) {
	if (invert==1){
	TM_ILI9341_SetCursorPosition(x1, y1, x2, y2);
	TM_ILI9341_SendCommand(0x39);
	}
	else
	{
	TM_ILI9341_SetCursorPosition(x1, y1, x2, y2);
	//TM_ILI9341_SendCommand(0x20);
	}

}

//dacus 16bit and speed
void Fill(uint32_t color) {
	uint16_t n;
	uint16_t j;
	uint32_t color2;
	color2 = ((color << 8)|(color >> 8));
	for (j=0; j<ILI9341_Opts.width;++j)
	{
	LCDbuffer[j]=color2;
	}

	for (n=0; n < ILI9341_Opts.height ; ++n){
		TM_ILI9341_SetCursorPosition(0, n, ILI9341_Opts.width -1 , ILI9341_Opts.height-1);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.width);

	}
}



//dacus fill2 with fade vertical
void Fill2(uint32_t color) {
	uint16_t n;
	uint16_t j;
	uint32_t color2;
	color2 = ((color << 8)|(color >> 8));
	for (j=0; j<ILI9341_Opts.width;++j)
	{
	LCDbuffer[j]=color2;
	}

	for (n=0; n < ILI9341_Opts.height ; n=n+4){
	    TM_ILI9341_SetCursorPosition(0, n, ILI9341_Opts.width -1, n);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.width);
	}
	for (n=2; n < ILI9341_Opts.height ; n=n+4){
		TM_ILI9341_SetCursorPosition(0, n, ILI9341_Opts.width -1, n);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.width);
		}
	for (n=1; n < ILI9341_Opts.height ; n=n+4){
		TM_ILI9341_SetCursorPosition(0, n, ILI9341_Opts.width -1, n);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.width);
			}
	for (n=3; n < ILI9341_Opts.height ; n=n+4){
		TM_ILI9341_SetCursorPosition(0, n, ILI9341_Opts.width -1, n);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.width);
				}

}

//dacus fill2 with fade vertical
void Fill_menu(uint32_t color) {
	uint16_t n;
	uint16_t j;
	uint32_t color2;
	color2 = ((color << 8)|(color >> 8));
	for (j=0; j<ILI9341_Opts.width;++j)
	{
	LCDbuffer[j]=color2;
	}

	for (n=0; n < ILI9341_Opts.height-60 ; n=n+4){
	    TM_ILI9341_SetCursorPosition(0, n, ILI9341_Opts.width -1, n);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.width);
	}
	for (n=2; n < ILI9341_Opts.height-60 ; n=n+4){
		TM_ILI9341_SetCursorPosition(0, n, ILI9341_Opts.width -1, n);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.width);
		}
	for (n=1; n < ILI9341_Opts.height-60 ; n=n+4){
		TM_ILI9341_SetCursorPosition(0, n, ILI9341_Opts.width -1, n);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.width);
			}
	for (n=3; n < ILI9341_Opts.height-60 ; n=n+4){
		TM_ILI9341_SetCursorPosition(0, n, ILI9341_Opts.width -1, n);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.width);
				}

}

void Fill_sub_menu(uint32_t color) {
	uint16_t n;
	uint16_t j;
	uint32_t color2;
	color2 = ((color << 8)|(color >> 8));
	for (j=0; j<ILI9341_Opts.width-240;++j)
	{
	LCDbuffer[j]=color2;
	}

	for (n=20; n < ILI9341_Opts.height-60 ; n=n+4){
	    TM_ILI9341_SetCursorPosition(0, n, ILI9341_Opts.width -241, n);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.width-240);
	}
	for (n=22; n < ILI9341_Opts.height-60 ; n=n+4){
		TM_ILI9341_SetCursorPosition(0, n, ILI9341_Opts.width -241, n);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.width-240);
		}
	for (n=21; n < ILI9341_Opts.height-60 ; n=n+4){
		TM_ILI9341_SetCursorPosition(0, n, ILI9341_Opts.width -241, n);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.width-240);
			}
	for (n=23; n < ILI9341_Opts.height-60 ; n=n+4){
		TM_ILI9341_SetCursorPosition(0, n, ILI9341_Opts.width -241, n);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.width-240);
				}

}




//dacus fill2 with fade horizontal
void Fill3(uint32_t color) {
	uint16_t n;
	uint16_t j;
	uint32_t color2;
	color2 = ((color << 8)|(color >> 8));
	for (j=0; j<ILI9341_Opts.height;++j)
	{
	LCDbuffer[j]=color2;
	}

	for (n=0; n < ILI9341_Opts.width ; n=n+4){
	    TM_ILI9341_SetCursorPosition(n, 0, n, ILI9341_Opts.height-1);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.height);
	}
	for (n=2; n < ILI9341_Opts.width ; n=n+4){
		TM_ILI9341_SetCursorPosition(n,0, n, ILI9341_Opts.height-1);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.height);
		}
	for (n=1; n < ILI9341_Opts.width ; n=n+4){
		TM_ILI9341_SetCursorPosition(n, 0, n, ILI9341_Opts.height-1);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.height);
			}
	for (n=3; n < ILI9341_Opts.width ; n=n+4){
		TM_ILI9341_SetCursorPosition(n, 0, n, ILI9341_Opts.height-1);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		//SendData16_fill(color,ILI9341_Opts.width);
		TM_ILI9341_SendData16_multi(LCDbuffer ,ILI9341_Opts.height);
				}

}


//---------------------------------------------------------------------------------------

void extconnector(uint16_t x0, uint16_t y0,uint8_t textcolor)
{
	uint16_t n;

	DrawHline(0,320,y0-30,bmscolor[textcolor][25]);

	DrawHline(x0+138, x0+183,  y0-7, bmscolor[0][55]);
	DrawHline(x0+140, x0+181,  y0-5, bmscolor[0][55]);

	DrawHline(x0+138, x0+184,  y0+142, bmscolor[0][55]);
	DrawHline(x0+140, x0+182,  y0+140, bmscolor[0][55]);

	DrawVline(y0-7,  y0+54,   x0+138, bmscolor[0][55]);
	DrawVline(y0+81, y0+142,  x0+138, bmscolor[0][55]);
	DrawVline(y0-7,  y0+142,  x0+183, bmscolor[0][55]);

	DrawVline(y0-5,  y0+54,   x0+140, bmscolor[0][55]);
	DrawVline(y0+81, y0+140,  x0+140, bmscolor[0][55]);
	DrawVline(y0-5,  y0+140,  x0+181, bmscolor[0][55]);

	DrawHline(x0+138, x0+140,  y0+53, bmscolor[0][55]);
	DrawHline(x0+138, x0+140,  y0+81, bmscolor[0][55]);


	for (n=0; n < 10 ; n++)	{ WriteDec(x0+144,y0+(n*14), &TM_Font_7x10, bmscolor[0][63] ,bmscolor[0][15], 2,0,(n*2)+1,0); Puts(x0+50,  y0+(n*14), extconnector1[(n*2)],   &TM_Font_7x10, bmscolor[textcolor][60],bmscolor[0][5]); }
	for (n=0; n < 10; n++)	{ WriteDec(x0+164,y0+(n*14), &TM_Font_7x10, bmscolor[0][63] ,bmscolor[0][15], 2,0,n+n+2,0);   Puts(x0+188, y0+(n*14), extconnector1[(n+n+1)], &TM_Font_7x10, bmscolor[textcolor][60],bmscolor[0][5]); }

}


//----------------------------------------------------------------

void cellwire(uint16_t x0, uint16_t y0,uint8_t textcolor)
{
	uint16_t n;

	DrawHline(0,320,y0,bmscolor[textcolor][25]);



	DrawHline(x0+183, x0+265,  y0+5, bmscolor[0][55]);
	DrawHline(x0+185, x0+263,  y0+7, bmscolor[0][55]);

	DrawHline(x0+185, x0+215,  y0+35, bmscolor[0][55]);
	DrawHline(x0+183, x0+215,  y0+37, bmscolor[0][55]);

	DrawHline(x0+233, x0+263,  y0+35, bmscolor[0][55]);
	DrawHline(x0+233, x0+265,  y0+37, bmscolor[0][55]);

	DrawVline(y0+35, y0+37,  x0+214, bmscolor[0][55]);
	DrawVline(y0+35, y0+37,  x0+233, bmscolor[0][55]);

	DrawVline(y0+7, y0+36,  x0+185, bmscolor[0][55]);
	DrawVline(y0+7, y0+36,  x0+263, bmscolor[0][55]);
	DrawVline(y0+5, y0+38,  x0+183, bmscolor[0][55]);
	DrawVline(y0+5, y0+38,  x0+265, bmscolor[0][55]);

	for (n=0; n < 10 ; n++)
	{
	if (n<6) {DrawVline(y0+40, y0+54+(n*5),  x0+200+(n*5), wirecolor[9-n]); DrawVline(y0+53+(n*5), y0+102,  x0+(n*40), wirecolor[9-n]); DrawHline(x0+(n*40), x0+200+(n*5),  y0+53+(n*5), wirecolor[9-n]);}
    if ((n>=6)&&(n<9)){DrawVline(y0+40, y0+79-((n-6)*5),  x0+200+(n*5), wirecolor[9-n]);DrawVline(y0+78-((n-6)*5), y0+102,  x0+(n*40), wirecolor[9-n]); DrawHline(x0+200+(n*5), x0+(n*40), y0+78-((n-6)*5), wirecolor[9-n]);}
	if (n==9){DrawVline(y0+40, y0+79-((n-6)*5),  x0+200+(n*5), wirecolor[9-n]);DrawVline(y0+78-((n-6)*5), y0+78-((n-6)*5)+5,  x0+((n-1)*40), wirecolor[9-n]); DrawHline(x0+200+(n*5), x0+((n-1)*40), y0+78-((n-6)*5), wirecolor[9-n]);}
	}


	for (n=0; n < 5 ; n++)	{WriteDec(x0+240-(n*10),y0+10, &TM_Font_7x10, bmscolor[0][5] ,wirecolor[n*2], 1,0,(n*2)+1,0);}
	for (n=0; n < 4; n++)	{WriteDec(x0+240-(n*10),y0+24, &TM_Font_7x10, bmscolor[0][5] ,wirecolor[n+n+1], 1,0,n+n+2,0);}
	Puts(x0+200, y0+24, "+", &TM_Font_7x10, bmscolor[0][5],wirecolor[9]);


}

void smallcell(uint16_t x0, uint16_t y0, uint8_t numberx,uint8_t cell1)
{

	uint16_t n;
	uint16_t j;

if (cell1==1){
	for (n=0; n < 10 ; n++){
    j=n*2;
	DrawHline(x0, x0+2,  y0+n+4, bmscolor[0][28+j]);
	}
	for (n=0; n < 18 ; n++){
	j=n*1.6;
	DrawHline(x0+2, x0+4,  y0+n, bmscolor[0][32+j]);
	DrawHline(x0+37, x0+39,  y0+n, bmscolor[0][32+j]);
	}

	ILI9341_x = x0+4;
	ILI9341_y = y0;
	LCDmenu_write_string_neg("+",6);
	LCDmenu_write_Dec1_neg(1,0,numberx,9);
	LCDmenu_write_string_neg("-",9);
	DrawHline(x0+2, x0+39,  y0-1, bmscolor[0][32-1]);
	DrawHline(x0+2, x0+39,  y0+18, bmscolor[0][50]);
	DrawHline(x0+39, x0+40,  y0+9, bmscolor[0][50]);
}
else
{
DrawFilledRectangle(x0,y0-1,x0+39,y0+19,0x0000);
DrawHline(x0, x0+40,  y0+9, bmscolor[0][50]);
DrawVline(y0-1, y0+11,  x0, wirecolor[numberx+1]);
DrawVline(y0+8, y0+11,  x0-1, wirecolor[numberx+1]);
DrawVline(y0+8, y0+11,  x0+1, wirecolor[numberx+1]);
}


}






//------------------------------------------------------------------------------------------------------------------------------

void EnergyGraph(uint16_t x0, uint16_t y0,uint32_t color,uint32_t color1,uint8_t textcolor,uint32_t *value,uint8_t val,uint32_t *value1,uint8_t val1,uint16_t *vbatt,uint8_t val3,char *str,uint8_t data1,uint8_t graphx,uint8_t graphy,uint8_t cursor) {
	uint16_t n;
	uint16_t j;
	uint32_t max1;
	float div1;
	uint32_t cursor1,cursor2,cursor3,cursor4,cursor5;
	uint8_t value2[241],value3[241];
	uint32_t color2;
	uint32_t color3;
	color2 = ((color << 8)|(color >> 8));
    color3 = ((color1 << 8)|(color1 >> 8));
    uint8_t difx,p,p1;
    difx=data1-graphx;

    p=0;
    p1=0;
    max1=0;
    cursor2=0;
    cursor4=0;
    cursor5=0;

if (value[data1-1]<11000000)//positive values
    {

if (val3==1) {	for (n=0; n < graphx ; n++){if (  max1<=((value[n+difx]*val)+(value1[n+difx]*val1))*(vbatt[n+difx]/100)   ){  max1=((value[n+difx]*val)+(value1[n+difx]*val1))*(vbatt[n+difx]/100)  ;}}
div1=(max1/graphy)+1;
if (div1<1000){div1=1000;}
if (cursor!=0) {cursor2=((value[cursor+difx-1]*val)+(value1[cursor+difx-1]*val1))*(vbatt[cursor+difx-1]/100);cursor4=(value[cursor+difx-1]*val)*(vbatt[cursor+difx-1]/100);}
}
else         {	for (n=0; n < graphx ; n++){if (  max1<=((value[n+difx]*val)+(value1[n+difx]*val1)) ){  max1=((value[n+difx]*val)+(value1[n+difx]*val1)); }}
div1=(max1/graphy)+1;
if (div1<2){div1=2;}
if (cursor!=0) {cursor2=((value[cursor+difx-1]*val)+(value1[cursor+difx-1]*val1));cursor4=(value[cursor+difx-1]*val);}
}



if (val3==1) {	for (n=0; n < graphx ; n++){value2[n]=((value[n+difx]*(vbatt[n+difx]/100))/div1)*val;  value3[n]=( ((value1[n+difx]+value[n+difx])*(vbatt[n+difx]/100)) /div1)*val1;} }
else         {  for (n=0; n < graphx ; n++){value2[n]=((value[n+difx])/div1)*val;  value3[n]=( (value1[n+difx]+value[n+difx])/div1)*val1;} }

cursor1=max1/div1;
cursor3=cursor2/div1;
cursor5=cursor4/div1;

    }

else  //positive and negative
    {
if (val3==1)
{
	for (n=0; n < graphx ; n++)
    {
    if (value[n+difx]>=12000000){if(  max1<=((value[n+difx]-12000000)*(vbatt[n+difx]/100))   ){max1=((value[n+difx]-12000000)*(vbatt[n+difx]/100));}  }
    else                       {if(  max1<=((12000000-value[n+difx])*(vbatt[n+difx]/100))   ){max1=((12000000-value[n+difx])*(vbatt[n+difx]/100));}  }
    }
	if (cursor!=0) {
	if (value[cursor+difx-1]>=12000000) {cursor2=((value[cursor+difx-1]-12000000)*(vbatt[cursor+difx-1]/100));}
	else                               {cursor2=((12000000-value[cursor+difx-1])*(vbatt[cursor+difx-1]/100));}
	}

div1=(max1/graphy)+1;
if (div1<1000){div1=1000;}
cursor3=cursor2/div1;
}
else
{
	for (n=0; n < graphx ; n++)
	 {
	 if (value[n+difx]>=12000000){if(  max1<=(value[n+difx]-12000000)   ){max1=(value[n+difx]-12000000);} }
	 else                       {if(  max1<=(12000000-value[n+difx])   ){max1=(12000000-value[n+difx]);} }
	 }
	if (cursor!=0) {
	if (value[cursor+difx-1]>=12000000){cursor2=(value[cursor+difx-1]-12000000);}
	else                              {cursor2=(12000000-value[cursor+difx-1]);}
	}
div1=(max1/graphy)+1;
if (div1<2){div1=2;}
cursor3=cursor2/div1;
}


if (val3==1)
{
    for (n=0; n < graphx ; n++)
    {
    if (value[n+difx]>=12000000){value2[n]=(((value[n+difx]-12000000)*(vbatt[n+difx]/100))/div1);value3[n]=0;}
    else                       {value3[n]=(((12000000-value[n+difx])*(vbatt[n+difx]/100))/div1);value2[n]=0;}
    }
}
else
{
    for (n=0; n < graphx ; n++)
    {
    if (value[n+difx]>=12000000){value2[n]=((value[n+difx]-12000000)/div1);value3[n]=0;}
    else                       {value3[n]=((12000000-value[n+difx])/div1);value2[n]=0;}
    }
}

cursor1=max1/div1;


    }

	for (n=0; n < graphx ; ++n){
	for (j=graphy; j>0;--j)
	{


	if      ((graphy-value2[n])<j){LCDbuffer[j]=color2;}
	else if ((graphy-value3[n])<j){LCDbuffer[j]=color3;}
	else if (graphy-cursor1==j)   {LCDbuffer[j]=11222;}
	else                          {LCDbuffer[j]=0xFFFF;}
	if ((cursor!=0)&((n==59)||(n==119)||(n==179))&(p1==0)){LCDbuffer[j]=12222;p1++;}
	else {if (p1>=1){p1=0;}}
	if ((cursor!=0)&(n==cursor-1)&((p==0)||(p==1))){LCDbuffer[j]=0x0000;p++;}
	else {if (p>=2){p=0;}}
	if ((cursor!=0)&(graphy-cursor3==j)){LCDbuffer[j]=0x0000;}
	if ((cursor!=0)&(graphy-cursor5==j)){LCDbuffer[j]=2112;}

	}
	    TM_ILI9341_SetCursorPosition(n+x0, 0+y0, n+x0, graphy+y0);
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		TM_ILI9341_SendData16_multi(LCDbuffer ,graphy);
	}




	if (cursor!=0)
	{
	if ((cursor>3)&(cursor<120))	{WriteDec(x0+cursor,y0+137, &TM_Font_7x10, 0xFFFE, bmscolor[textcolor][0], 3,0,240-cursor,1);}
	if (cursor<=3)              	{WriteDec(x0+4,y0+137, &TM_Font_7x10, 0xFFFE, bmscolor[textcolor][0], 3,0,240-cursor,1);}
	if ((cursor>=120)&(cursor<237))	{WriteDec(x0+cursor-42,y0+137, &TM_Font_7x10, 0xFFFE, bmscolor[textcolor][0], 3,0,240-cursor,1);}
	if (cursor>=237)				{WriteDec(x0+194,y0+137, &TM_Font_7x10, 0xFFFE, bmscolor[textcolor][0], 3,0,240-cursor,1);}


	for (j=0; j < 8 ; j++)
	{
    if ((cursor>3)&(cursor<237))
    {
    DrawHline(x0, x0+cursor-4,  y0+j+126, bmscolor[textcolor][60-(j*4)]);
    DrawHline(x0+cursor-4, x0+cursor+4,  y0+j+126, bmscolor[textcolor][32-(j*3)]);
	DrawHline(x0+cursor+4, x0+graphx,  y0+j+126, bmscolor[textcolor][60-(j*4)]);


    if ((cursor>3)&(cursor<120)){
	DrawHline(x0, x0+cursor,  y0+j+137, bmscolor[textcolor][32-(j*3)]);
	DrawHline(x0+cursor+42, x0+graphx,  y0+j+137, bmscolor[textcolor][32-(j*3)]);
	}

	if ((cursor>=120)&(cursor<237)){
	DrawHline(x0, x0+cursor-42,  y0+j+137, bmscolor[textcolor][32-(j*3)]);
	DrawHline(x0+cursor, x0+graphx,  y0+j+137, bmscolor[textcolor][32-(j*3)]);
	}

    }
    else if (cursor<3)
    {
    DrawHline(x0, x0,  y0+j+126, bmscolor[textcolor][60-(j*4)]);
    DrawHline(x0, x0+8,  y0+j+126, bmscolor[textcolor][32-(j*3)]);
    DrawHline(x0+8, x0+graphx,  y0+j+126, bmscolor[textcolor][60-(j*4)]);
    }
    else if (cursor>237)
    {
    DrawHline(x0, x0+232,  y0+j+126, bmscolor[textcolor][60-(j*4)]);
    DrawHline(x0+232, x0+240,  y0+j+126, bmscolor[textcolor][32-(j*3)]);
    DrawHline(x0+240, x0+graphx,  y0+j+126, bmscolor[textcolor][60-(j*4)]);
    }

    if (cursor<=3){
    DrawHline(x0, x0+4,  y0+j+137, bmscolor[textcolor][32-(j*3)]);
    DrawHline(x0+46, x0+graphx,  y0+j+137, bmscolor[textcolor][32-(j*3)]);
    }
	if (cursor>=237){
	DrawHline(x0, x0+194,  y0+j+137, bmscolor[textcolor][32-(j*3)]);
	DrawHline(x0+236, x0+graphx,  y0+j+137, bmscolor[textcolor][32-(j*3)]);
	}


    }
	//DrawHline(x0+cursor-4, x0+cursor+4,  y0+125, bmscolor[textcolor][35]);
	//DrawHline(x0+cursor-4, x0+cursor+4,  y0+134, bmscolor[textcolor][12]);



if (val3!=0)
{
cursor2=cursor2/1000;
cursor4=cursor4/1000;
}



	if (value[data1-1]<11000000)
	{

	{Puts(0, 60, " ", &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5]);}
	{Puts(0, 100, " ", &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5]);}

	if (val3==0)
	{
	if      ((cursor2-cursor4)>=100000){WriteDec(11, 60, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 5,2,(cursor2-cursor4)/10,1);}
	else if ((cursor2-cursor4)>=10000) {WriteDec(11, 60, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 5,3,cursor2-cursor4,1);}
	else                               {WriteDec(11, 60, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 4,3,cursor2-cursor4,1);}
	}

	else{
	if  ((cursor2-cursor4)<100)                                 {WriteDec(11, 60, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 2,1,cursor2-cursor4,1);}
	if (((cursor2-cursor4)>=100)&((cursor2-cursor4)<1000))      {WriteDec(11, 60, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 3,1,cursor2-cursor4,1);}
	if (((cursor2-cursor4)>=1000)&((cursor2-cursor4)<10000))    {WriteDec(11, 60, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 4,1,cursor2-cursor4,1);}
	if (((cursor2-cursor4)>=10000)&((cursor2-cursor4)<100000))  {WriteDec(11, 60, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 5,1,cursor2-cursor4,1);}
	if (((cursor2-cursor4)>=100000)&((cursor2-cursor4)<1000000)){WriteDec(11, 60, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 5,0,(cursor2-cursor4)/10,1);}
	}



	if (val3==0)
	{
	if      (cursor4>=100000){WriteDec(11, 100, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 5,2,cursor4/10,1);}
	else if (cursor4>=10000) {WriteDec(11, 100, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 5,3,cursor4,1);}
	else                     {WriteDec(11, 100, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 4,3,cursor4,1);}
	}

	else{
	if  (cursor4<100)                       {WriteDec(11, 100, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 2,1,cursor4,1);}
	if ((cursor4>=100)&(cursor4<1000))      {WriteDec(11, 100, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 3,1,cursor4,1);}
	if ((cursor4>=1000)&(cursor4<10000))    {WriteDec(11, 100, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 4,1,cursor4,1);}
	if ((cursor4>=10000)&(cursor4<100000))  {WriteDec(11, 100, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 5,1,cursor4,1);}
	if ((cursor4>=100000)&(cursor4<1000000)){WriteDec(11, 100, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 5,0,cursor4/10,1);}
	}
}





	if       (value[cursor+difx-1]>=12000000)                                  {Puts(0, 140, "+", &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5]);}
	else if ((value[cursor+difx-1]<12000000)&&(value[cursor+difx-1]>11000000)) {Puts(0, 140, "-", &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5]);}
	else                                                                       {Puts(0, 140, " ", &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5]);}




	if (val3==0)
	{
	if      (cursor2>=100000) {WriteDec(11, 140, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 5,2,cursor2/10,1);}
	else if (cursor2>=10000)  {WriteDec(11, 140, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 5,3,cursor2,1);}
	else                      {WriteDec(11, 140, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 4,3,cursor2,1);}
	}

	else{
	if  (cursor2<100)                       {WriteDec(11, 140, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 2,1,cursor2,1);}
	if ((cursor2>=100)&(cursor2<1000))      {WriteDec(11, 140, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 3,1,cursor2,1);}
	if ((cursor2>=1000)&(cursor2<10000))    {WriteDec(11, 140, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 4,1,cursor2,1);}
	if ((cursor2>=10000)&(cursor2<100000))  {WriteDec(11, 140, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 5,1,cursor2,1);}
	if ((cursor2>=100000)&(cursor2<1000000)){WriteDec(11, 140, &TM_Font_11x18, bmscolor[textcolor][62], bmscolor[textcolor][5], 5,0,cursor2/10,1);}
	}




	}

	Puts(x0, y0-10, str, &TM_Font_7x10, 0xFFF5, bmscolor[textcolor][24]);

	if (val3==0)
	{
	if      (max1>=100000)   {WriteDec(x0+(graphx-42),y0-10, &TM_Font_7x10, 0xFFFE, bmscolor[textcolor][24], 5,2,max1/10,1);}
	else if (max1>=10000)    {WriteDec(x0+(graphx-42),y0-10, &TM_Font_7x10, 0xFFFE, bmscolor[textcolor][24], 5,3,max1,1);}
	else                     {WriteDec(x0+(graphx-42),y0-10, &TM_Font_7x10, 0xFFFE, bmscolor[textcolor][24], 4,3,max1,1);}
	}

	else
	{
	max1=max1/1000;
	if  (max1<100)                    {WriteDec(x0+(graphx-42),y0-10, &TM_Font_7x10, 0xFFFE, bmscolor[textcolor][24], 2,1,max1,1);}
	if ((max1>=100)&(max1<1000))      {WriteDec(x0+(graphx-42),y0-10, &TM_Font_7x10, 0xFFFE, bmscolor[textcolor][24], 3,1,max1,1);}
	if ((max1>=1000)&(max1<10000))    {WriteDec(x0+(graphx-42),y0-10, &TM_Font_7x10, 0xFFFE, bmscolor[textcolor][24], 4,1,max1,1);}
	if ((max1>=10000)&(max1<100000))  {WriteDec(x0+(graphx-42),y0-10, &TM_Font_7x10, 0xFFFE, bmscolor[textcolor][24], 5,1,max1,1);}
	if ((max1>=100000)&(max1<1000000)){WriteDec(x0+(graphx-42),y0-10, &TM_Font_7x10, 0xFFFE, bmscolor[textcolor][24], 5,0,max1/10,1);}
	}


}


//--------------------------------------------------------------------------------------------------------------


void TM_ILI9341_Delay(volatile unsigned int delay) {
	for (; delay != 0; delay--);
}

void TM_ILI9341_Rotate(TM_ILI9341_Orientation_t orientation) {
	TM_ILI9341_SendCommand(ILI9341_MAC);
	if (orientation == TM_ILI9341_Orientation_Portrait_1) {
		TM_ILI9341_SendData(0x58);
	} else if (orientation == TM_ILI9341_Orientation_Portrait_2) {
		TM_ILI9341_SendData(0x88);
	} else if (orientation == TM_ILI9341_Orientation_Landscape_1) {
		TM_ILI9341_SendData(0x28);
	} else if (orientation == TM_ILI9341_Orientation_Landscape_2) {
		TM_ILI9341_SendData(0xE8);
	}

	if (orientation == TM_ILI9341_Orientation_Portrait_1 || orientation == TM_ILI9341_Orientation_Portrait_2) {
		ILI9341_Opts.width = ILI9341_WIDTH;
		ILI9341_Opts.height = ILI9341_HEIGHT;
		ILI9341_Opts.orientation = TM_ILI9341_Portrait;
	} else {
		ILI9341_Opts.width = ILI9341_HEIGHT;
		ILI9341_Opts.height = ILI9341_WIDTH;
		ILI9341_Opts.orientation = TM_ILI9341_Landscape;
	}
}

void Puts(uint16_t x, uint16_t y, char *str, TM_FontDef_t *font, uint32_t foreground, uint32_t background) {
	uint16_t startX = x;

	/* Set X and Y coordinates */
	ILI9341_x = x;
	ILI9341_y = y;

	while (*str) {
		//New line
		if (*str == '\n') {
			ILI9341_y += font->FontHeight + 1;
			//if after \n is also \r, than go to the left of the screen
			if (*(str + 1) == '\r') {
				ILI9341_x = 0;
				str++;
			} else {
				ILI9341_x = startX;
			}
			str++;
			continue;
		} else if (*str == '\r') {
			str++;
			continue;
		}

		Putc(ILI9341_x, ILI9341_y, *str++, font, foreground, background);
	}
}

void Puts2(uint16_t x, uint16_t y, char *str, TM_FontDef_t *font, uint32_t foreground, uint32_t background,uint8_t linespace) {
	uint16_t startX = x;

	/* Set X and Y coordinates */
	ILI9341_x = x;
	ILI9341_y = y;

	while (*str) {
		//New line
		if (*str == '\n') {
			ILI9341_y += font->FontHeight + linespace;
			//if after \n is also \r, than go to the left of the screen
			if (*(str + 1) == '\r') {
				ILI9341_x = 0;
				str++;
			} else {
				ILI9341_x = startX;
			}
			str++;
			continue;
		} else if (*str == '\r') {
			str++;
			continue;
		}

		Putc(ILI9341_x, ILI9341_y, *str++, font, foreground, background);
	}
}


void TM_ILI9341_GetStringSize(char *str, TM_FontDef_t *font, uint16_t *width, uint16_t *height) {
	uint16_t w = 0;
	*height = font->FontHeight;
	while (*str++) {
		w += font->FontWidth;
	}
	*width = w;
}

//dacus speed increase using character buffer and sending just the color data and the coordinate for the start pixel.
void Putc(uint16_t x, uint16_t y, char c, TM_FontDef_t *font, uint32_t foreground, uint32_t background) {
	uint32_t i, b, j;

	//uint32_t lcd;

    /* Set coordinates */
	ILI9341_x = x;
	ILI9341_y = y;
	if ((ILI9341_x + font->FontWidth) > ILI9341_Opts.width) {
		//If at the end of a line of display, go to new line and set x to 0 position
		ILI9341_y += font->FontHeight;
		ILI9341_x = 0;
	}
	for (i = 0; i < font->FontHeight; i++) {
		b = font->data[(c - 32) * font->FontHeight + i];

		for (j = 0; j < font->FontWidth; j++) {
			if ((b << j) & 0x8000) {
				LCDbuffer[j+(i*font->FontWidth)]=((foreground << 8)|(foreground >> 8));
			} else if ((background & ILI9341_TRANSPARENT) == 0) {
				LCDbuffer[j+(i*font->FontWidth)]=((background << 8)|(background >> 8));
			}
		}
	}

	TM_ILI9341_SetCursorPosition(ILI9341_x , (ILI9341_y ), ILI9341_x + font->FontWidth-1, (ILI9341_y+font->FontHeight-1));
	TM_ILI9341_SendCommand(ILI9341_GRAM);
	TM_ILI9341_SendCommand(ILI9341_WRITE_CONTINUE);
	TM_ILI9341_SendData16_multi(LCDbuffer ,(font->FontWidth*font->FontHeight));
	ILI9341_x += font->FontWidth;
}

//dacus
void DrawHline(uint16_t x0, uint16_t x1, uint16_t y0, uint32_t color){
	if ((x1>x0)&(x1<=320)){
	uint16_t j;
	uint32_t color2;

	color2 = ((color << 8)|(color >> 8));
	for (j=0; j<(x1-x0);++j)
	{
	LCDbuffer[j]=color2;
	}

	TM_ILI9341_SetCursorPosition(x0, y0, x1+1, y0);
	TM_ILI9341_SendCommand(ILI9341_GRAM);
	TM_ILI9341_SendCommand(ILI9341_WRITE_CONTINUE);
	TM_ILI9341_SendData16_multi(LCDbuffer ,(x1-x0));

	//SendData16_fill(color ,(x1-x0));
}
}

//dacus
void DrawVline(uint16_t y0,uint16_t y1, uint16_t x0, uint32_t color){
	if ((y1>y0)&(y1<=240)){
		uint16_t j;
		uint32_t color2;

		color2 = ((color << 8)|(color >> 8));
		for (j=0; j<(y1-y0);++j)
		{
		LCDbuffer[j]=color2;
		}

	TM_ILI9341_SetCursorPosition(x0, y0, x0, y1+1);
	TM_ILI9341_SendCommand(ILI9341_GRAM);
	TM_ILI9341_SendCommand(ILI9341_WRITE_CONTINUE);
	TM_ILI9341_SendData16_multi(LCDbuffer ,(y1-y0));
	//SendData16_fill(color ,(y1-y0));
}
}



//dacus
void DrawFilledRectangle(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1, uint32_t color){
	if ((x1>=x0)&&(y1>=y0)){
	uint16_t n;
	uint16_t j;
	uint32_t color2;
	color2 = ((color << 8)|(color >> 8));
	for (j=0; j<(x1-x0);++j)
	{
	LCDbuffer[j]=color2;
	}

	TM_ILI9341_SetCursorPosition(x0, y0, x1-1, y1-1);
	TM_ILI9341_SendCommand(ILI9341_GRAM);
	for (n=0; n < (y1-y0) ; ++n){
	TM_ILI9341_SendData16_multi(LCDbuffer ,(x1-x0));
    }
	}
}



void TM_ILI9341_DrawLine(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1, uint32_t color) {
	/* Code by dewoller: https://github.com/dewoller */

	int16_t dx, dy, sx, sy, err, e2;

	/* Check for overflow */
	if (x0 >= ILI9341_Opts.width) {
		x0 = ILI9341_Opts.width - 1;
	}
	if (x1 >= ILI9341_Opts.width) {
		x1 = ILI9341_Opts.width - 1;
	}
	if (y0 >= ILI9341_Opts.height) {
		y0 = ILI9341_Opts.height - 1;
	}
	if (y1 >= ILI9341_Opts.height) {
		y1 = ILI9341_Opts.height - 1;
	}

	dx = (x0 < x1) ? (x1 - x0) : (x0 - x1);
	dy = (y0 < y1) ? (y1 - y0) : (y0 - y1);
	sx = (x0 < x1) ? 1 : -1;
	sy = (y0 < y1) ? 1 : -1;
	err = ((dx > dy) ? dx : -dy) / 2;

	while (1) {
		TM_ILI9341_DrawPixel(x0, y0, color);
		if (x0 == x1 && y0 == y1) {
			break;
		}
		e2 = err;
		if (e2 > -dx) {
			err -= dy;
			x0 += sx;
		}
		if (e2 < dy) {
			err += dx;
			y0 += sy;
		}
	}


}

void DrawRectangle(uint16_t x0, uint16_t y0, uint16_t x1, uint16_t y1, uint32_t color) {
	 DrawHline(x0, x1, y0, color); //Top
	 DrawVline(y0, y1, x0, color);	//Left
	 DrawVline(y0, y1+1, x1, color);	//Right
	 DrawHline(x0, x1, y1, color);	//Bottom
}

void TM_ILI9341_DrawCircle(int16_t x0, int16_t y0, int16_t r, uint32_t color) {
	int16_t f = 1 - r;
	int16_t ddF_x = 1;
	int16_t ddF_y = -2 * r;
	int16_t x = 0;
	int16_t y = r;

    TM_ILI9341_DrawPixel(x0, y0 + r, color);
    TM_ILI9341_DrawPixel(x0, y0 - r, color);
    TM_ILI9341_DrawPixel(x0 + r, y0, color);
    TM_ILI9341_DrawPixel(x0 - r, y0, color);

    while (x < y) {
        if (f >= 0) {
            y--;
            ddF_y += 2;
            f += ddF_y;
        }
        x++;
        ddF_x += 2;
        f += ddF_x;

        TM_ILI9341_DrawPixel(x0 + x, y0 + y, color);
        TM_ILI9341_DrawPixel(x0 - x, y0 + y, color);
        TM_ILI9341_DrawPixel(x0 + x, y0 - y, color);
        TM_ILI9341_DrawPixel(x0 - x, y0 - y, color);

        TM_ILI9341_DrawPixel(x0 + y, y0 + x, color);
        TM_ILI9341_DrawPixel(x0 - y, y0 + x, color);
        TM_ILI9341_DrawPixel(x0 + y, y0 - x, color);
        TM_ILI9341_DrawPixel(x0 - y, y0 - x, color);
    }
}

void DrawFilledCircle(int16_t x0, int16_t y0, int16_t r, uint32_t color) {
	int16_t f = 1 - r;
	int16_t ddF_x = 1;
	int16_t ddF_y = -2 * r;
	int16_t x = 0;
	int16_t y = r;

    TM_ILI9341_DrawPixel(x0, y0 + r, color);
    TM_ILI9341_DrawPixel(x0, y0 - r, color);
    TM_ILI9341_DrawPixel(x0 + r, y0, color);
    TM_ILI9341_DrawPixel(x0 - r, y0, color);
    DrawHline(x0 - r, x0 + r, y0, color);

    while (x < y) {
        if (f >= 0) {
            y--;
            ddF_y += 2;
            f += ddF_y;
        }
        x++;
        ddF_x += 2;
        f += ddF_x;

        DrawHline(x0 - x, x0 + x, y0 + y, color);
        DrawHline(x0 - x, x0 + x, y0 - y, color);
        DrawHline(x0 - y, x0 + y, y0 + x, color);
        DrawHline(x0 - y, x0 + y, y0 - x, color);

    }
}

void WriteDec(uint16_t x, uint16_t y ,TM_FontDef_t *font, uint32_t foreground, uint32_t background, uint8_t dig,uint8_t dec,uint32_t b,uint8_t c)
{

	char datas[4];
    uint8_t i,x1;
    x1=0;

  if (c!=0){
	if(dec!=0)
	{
	for (i=0;i<(6-(dig+1));i++){
	Putc(x+x1,y,32,font, foreground, background);
	x1=x1+font->FontWidth;
	}
	}
	else
	{
	for (i=0;i<(6-(dig));i++)
	{
	Putc(x+x1,y,32,font, foreground, background);
	x1=x1+font->FontWidth;
	}
	}
  }
  else{}


	if (dig >= 5)
		{
		datas[0] = b/10000;
		b = b - datas[0]*10000;
		datas[0]+=48;
		Putc(x+x1,y,datas[0],font, foreground, background);
		x1=x1+font->FontWidth;
		if (dec==4)
		{
		Putc(x+x1,y,46,font, foreground, background);
		x1=x1+font->FontWidth;
		}
		}


	if (dig >= 4)
	{
	datas[0] = b/1000;
	b = b - datas[0]*1000;
	datas[0]+=48;
	Putc(x+x1,y,datas[0],font, foreground, background);
	x1=x1+font->FontWidth;
	if (dec==3)
	{
	Putc(x+x1,y,46,font, foreground, background);
	x1=x1+font->FontWidth;
	}
	}

	if (dig >= 3)
		{
		datas[1] = b/100;
		b = b - datas[1]*100;
		datas[1]+=48;
		Putc(x+x1,y,datas[1],font, foreground, background);
		x1=x1+font->FontWidth;
		if (dec==2)
		{
		Putc(x+x1,y,46,font, foreground, background);
		x1=x1+font->FontWidth;
		}
		}

	if (dig >= 2)
		{
		datas[2] = b/10;
		b = b - datas[2]*10;
		datas[2]+=48;
		Putc(x+x1,y,datas[2],font, foreground, background);
		x1=x1+font->FontWidth;
		if (dec==1)
		{
		Putc(x+x1,y,46,font, foreground, background);
		x1=x1+font->FontWidth;
		}
		}

	datas[3] = b;

	datas[3]+=48;

	Putc(x+x1,y,datas[3],font, foreground, background);

}

void WriteDec2(uint16_t x, uint16_t y ,TM_FontDef_t *font, uint8_t menucolor, uint8_t dig,uint8_t dec,uint32_t b,uint8_t c,uint8_t neg)
{

	char datas[4];
    uint8_t i,x1;
    x1=0;

  if (c!=0){
	if(dec!=0)
	{
	for (i=0;i<(6-(dig+1));i++){
	Putc2(x+x1,y,32,font, menucolor,neg);
	x1=x1+font->FontWidth;
	}
	}
	else
	{
	for (i=0;i<(6-(dig));i++)
	{
	Putc2(x+x1,y,32,font, menucolor,neg);
	x1=x1+font->FontWidth;
	}
	}
  }
  else{}


	if (dig >= 5)
		{
		datas[0] = b/10000;
		b = b - datas[0]*10000;
		datas[0]+=48;
		Putc2(x+x1,y,datas[0],font, menucolor,neg);
		x1=x1+font->FontWidth;
		if (dec==4)
		{
		Putc2(x+x1,y,46,font, menucolor,neg);
		x1=x1+font->FontWidth;
		}
		}


	if (dig >= 4)
	{
	datas[0] = b/1000;
	b = b - datas[0]*1000;
	datas[0]+=48;
	Putc2(x+x1,y,datas[0],font, menucolor,neg);
	x1=x1+font->FontWidth;
	if (dec==3)
	{
	Putc2(x+x1,y,46,font, menucolor,neg);
	x1=x1+font->FontWidth;
	}
	}

	if (dig >= 3)
		{
		datas[1] = b/100;
		b = b - datas[1]*100;
		datas[1]+=48;
		Putc2(x+x1,y,datas[1],font, menucolor,neg);
		x1=x1+font->FontWidth;
		if (dec==2)
		{
		Putc2(x+x1,y,46,font, menucolor,neg);
		x1=x1+font->FontWidth;
		}
		}

	if (dig >= 2)
		{
		datas[2] = b/10;
		b = b - datas[2]*10;
		datas[2]+=48;
		Putc2(x+x1,y,datas[2],font, menucolor,neg);
		x1=x1+font->FontWidth;
		if (dec==1)
		{
		Putc2(x+x1,y,46,font, menucolor,neg);
		x1=x1+font->FontWidth;
		}
		}

	datas[3] = b;

	datas[3]+=48;

	Putc2(x+x1,y,datas[3],font, menucolor,neg);




}



void Putc2(uint16_t x, uint16_t y, char c, TM_FontDef_t *font, uint8_t menucolor, uint8_t neg) {
	uint32_t i, b, j;


	uint32_t background;
	background=menubackground;


    /* Set coordinates */
	ILI9341_x = x;
	ILI9341_y = y;
	if ((ILI9341_x + font->FontWidth) > ILI9341_Opts.width) {
		//If at the end of a line of display, go to new line and set x to 0 position
		ILI9341_y += font->FontHeight;
		ILI9341_x = 0;
	}
	for (i = 0; i < font->FontHeight; i++) {
		b = font->data[(c - 32) * font->FontHeight + i];

		if (neg==0){
		for (j = 0; j < font->FontWidth; j++) {
			if ((b << j) & 0x8000) {
				LCDbuffer[j+(i*font->FontWidth)]=((bmscolor[menucolor][63] << 8)|(bmscolor[menucolor][63] >> 8));
					} else if ((background & ILI9341_TRANSPARENT) == 0) {
				LCDbuffer[j+(i*font->FontWidth)]=((bmscolor[menucolor][i/2] << 8)|(bmscolor[menucolor][i/2] >> 8));
			}
		}
		}
		else
		{
		for (j = 0; j < font->FontWidth; j++) {
			if ((b << j) & 0x8000) {
				LCDbuffer[j+(i*font->FontWidth)]=((bmscolor[menucolor][0] << 8)|(bmscolor[menucolor][0] >> 8));
					} else if ((background & ILI9341_TRANSPARENT) == 0) {
				LCDbuffer[j+(i*font->FontWidth)]=(bmscolor[menucolor][(i*2)+28] << 8)|(bmscolor[menucolor][(i*2)+28] >> 8);
			}
		}
		}


	}

	TM_ILI9341_SetCursorPosition(ILI9341_x , (ILI9341_y ), ILI9341_x + font->FontWidth-1, (ILI9341_y+font->FontHeight-1));
	TM_ILI9341_SendCommand(ILI9341_GRAM);
	TM_ILI9341_SendCommand(ILI9341_WRITE_CONTINUE);
	TM_ILI9341_SendData16_multi(LCDbuffer ,(font->FontWidth*font->FontHeight));
	ILI9341_x += font->FontWidth;
}






void LCDmenu_set_XY(unsigned char X,unsigned char Y)
{

	TM_FontDef_t *font;
	font = &TM_Font_11x18;
	ILI9341_x = X*font->FontWidth;
	ILI9341_y = Y*(font->FontHeight+2);

}

void LCDmenu_write_string(char *s,uint8_t menucolor)
{

	TM_FontDef_t *font;
	font = &TM_Font_11x18;

	uint16_t startX = ILI9341_x;

		while (*s) {
			//New line
			if (*s == '\n') {
				ILI9341_y += font->FontHeight + 1;
				//if after \n is also \r, than go to the left of the screen
				if (*(s + 1) == '\r') {
					ILI9341_x = 0;
					s++;
				} else {
					ILI9341_x = startX;
				}
				s++;
				continue;
			} else if (*s == '\r') {
				s++;
				continue;
			}

			LCDmenu_write_char(*s++,menucolor);
		}

}

void LCDmenu_write_char(unsigned char c,uint8_t menucolor)
{
	uint32_t i, b, j;
	TM_FontDef_t *font;

	uint32_t background;


	background=menubackground;


		//uint32_t lcd;
       font = &TM_Font_11x18;

	    /* Set coordinates */
		//ILI9341_x = x;
		//ILI9341_y = y;
		if ((ILI9341_x + font->FontWidth) > ILI9341_Opts.width) {
			//If at the end of a line of display, go to new line and set x to 0 position
			ILI9341_y += font->FontHeight;
			ILI9341_x = 0;
		}
		for (i = 0; i < font->FontHeight; i++) {
			b = font->data[(c - 32) * font->FontHeight + i];

			for (j = 0; j < font->FontWidth; j++) {
				if ((b << j) & 0x8000) {
					LCDbuffer[j+(i*font->FontWidth)]=((bmscolor[menucolor][60] << 8)|(bmscolor[menucolor][60] >> 8));
				} else if ((background & ILI9341_TRANSPARENT) == 0) {
					LCDbuffer[j+(i*font->FontWidth)]=((bmscolor[menucolor][i] << 8)|(bmscolor[menucolor][i] >> 8));
				}
			}
		}

		TM_ILI9341_SetCursorPosition(ILI9341_x , (ILI9341_y ), ILI9341_x + font->FontWidth-1, (ILI9341_y+font->FontHeight-1));
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		TM_ILI9341_SendCommand(ILI9341_WRITE_CONTINUE);
		TM_ILI9341_SendData16_multi(LCDbuffer ,(font->FontWidth*font->FontHeight));
		ILI9341_x += font->FontWidth;
}

void LCDmenu_write_string_neg(char *s,uint8_t menucolor)
{

	TM_FontDef_t *font;
	font = &TM_Font_11x18;

	uint16_t startX = ILI9341_x;

		while (*s) {
			//New line
			if (*s == '\n') {
				ILI9341_y += font->FontHeight + 1;
				//if after \n is also \r, than go to the left of the screen
				if (*(s + 1) == '\r') {
					ILI9341_x = 0;
					s++;
				} else {
					ILI9341_x = startX;
				}
				s++;
				continue;
			} else if (*s == '\r') {
				s++;
				continue;
			}

			LCDmenu_write_char_neg(*s++,menucolor);
		}

}



void LCDmenu_write_char_neg(unsigned char c,uint8_t menucolor)
{
	uint32_t i, b, j;
	TM_FontDef_t *font;

	uint32_t background;


	background=menuforeground;

		//uint32_t lcd;
       font = &TM_Font_11x18;

	    /* Set coordinates */
		//ILI9341_x = x;
		//ILI9341_y = y;
		if ((ILI9341_x + font->FontWidth) > ILI9341_Opts.width) {
			//If at the end of a line of display, go to new line and set x to 0 position
			ILI9341_y += font->FontHeight;
			ILI9341_x = 0;
		}
		for (i = 0; i < font->FontHeight; i++) {
			b = font->data[(c - 32) * font->FontHeight + i];

			for (j = 0; j < font->FontWidth; j++) {
				if ((b << j) & 0x8000) {
					LCDbuffer[j+(i*font->FontWidth)]=((bmscolor[menucolor][3] << 8)|(bmscolor[menucolor][3] >> 8));
				} else if ((background & ILI9341_TRANSPARENT) == 0) {
					LCDbuffer[j+(i*font->FontWidth)]=(bmscolor[menucolor][(i*2)+28] << 8)|(bmscolor[menucolor][(i*2)+28] >> 8);
				}
			}
		}

		TM_ILI9341_SetCursorPosition(ILI9341_x , (ILI9341_y ), ILI9341_x + font->FontWidth-1, (ILI9341_y+font->FontHeight-1));
		TM_ILI9341_SendCommand(ILI9341_GRAM);
		TM_ILI9341_SendCommand(ILI9341_WRITE_CONTINUE);
		TM_ILI9341_SendData16_multi(LCDbuffer ,(font->FontWidth*font->FontHeight));
		ILI9341_x += font->FontWidth;
}










void LCDmenu_write_Dec(unsigned int dig,unsigned int dec,unsigned int b,uint8_t menucolor)
{
	TM_FontDef_t *font;
	font = &TM_Font_11x18;

//WriteDec(ILI9341_x, ILI9341_y ,font, bmscolor[menucolor][63], bmscolor[menucolor][3], dig, dec, b,1);
WriteDec2(ILI9341_x, ILI9341_y ,font, menucolor, dig, dec, b,1,0);

}

void LCDmenu_write_Dec1(unsigned int dig,unsigned int dec,unsigned int b,uint8_t menucolor)
{
	TM_FontDef_t *font;
	font = &TM_Font_11x18;

//WriteDec(ILI9341_x, ILI9341_y ,font, bmscolor[menucolor][63], bmscolor[menucolor][3], dig, dec, b,0);
WriteDec2(ILI9341_x, ILI9341_y ,font, menucolor, dig, dec, b,0,0);

}

void LCDmenu_write_Dec_neg(unsigned int dig,unsigned int dec,unsigned int b,uint8_t menucolor)
{
	TM_FontDef_t *font;
	font = &TM_Font_11x18;

//WriteDec(ILI9341_x, ILI9341_y ,font, bmscolor[menucolor][3], bmscolor[menucolor][63], dig, dec, b,1);
WriteDec2(ILI9341_x, ILI9341_y ,font, menucolor, dig, dec, b,1,1);
}

void LCDmenu_write_Dec1_neg(unsigned int dig,unsigned int dec,unsigned int b,uint8_t menucolor)
{
	TM_FontDef_t *font;
	font = &TM_Font_11x18;

//WriteDec(ILI9341_x, ILI9341_y ,font, bmscolor[menucolor][3], bmscolor[menucolor][63], dig, dec, b,0);
WriteDec2(ILI9341_x, ILI9341_y ,font, menucolor, dig, dec, b,0,1);
}


